;                                               D93FUNCG.ASM
;
;  Digital sampling function generator for DSP-93 and D93WE Windows 
;  program  
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
;
; This document, and its executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
;
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
;
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
;
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;
;
;  Revision history
;
;  Version   Date     Author     Reason
;  ------- -------- -----------  ---------------------------------------------------------
;  0.3.5   10-17-94 TCMcDermott  Prototype #3 release
;  0.3.6   10-23-94 TCMcDermott  Change from regs.asm, monitor.asm to
;                                  regs.inc, monitor.inc
;  1.0     12-11-94 TCMcDermott  Production #1 release.
;                                Move page 8 memory references to avoid
;                                overlap with monitor 2.15.
;
;
;  This version programs the AIO for maximum sampling rate (62.5 ksps)
;  (40 Mhz CPU clock).
;  AIO initialization from Lon Cecil's BST5.ASM program
;
;  The algorithm used is:
;
;  1. Init the DSP-93 AIO and serial code 
;  2. loop: wait-for-RX-ser-char
;  3. D93WE will send 'P', frequency, amplitude, wave-type, and dummy
;  4.   if D93WE sends 'R', then RESET back to monitor.
;  4. DSP-93 goes into a loop sending the function pattern, and
;  5. DSP-93 scans for a serial character from D93WE.
;  6. If a character is received, goto 3.


#include        "regs.inc"
#include        "monitor.inc"

; ------------------ Internal memory usage page 0 -------------------
; ------------------ only valid for 60h-7Fh = B2  -------------------

; only using page 0 during the AIO configuration setup

BUFI      .EQU      7Ch       ;  hold input from AIO chip for INIT
BUFO      .EQU      7Dh       ;  hold output to AIO chip for INIT

; ------------------ Internal memory usage page 4 -------------------
; ---------  (200h-27Fh = B0 lower half when B0 is data)  -----------

; ------------------ Internal memory usage page 5 -------------------
; ---------  (280h-2FFh = B0 upper half when B0 is data)  -----------

; ------------------ Internal memory usage page 6 -------------------
; ------------------  (300h-37Fh = B1 lower half) -------------------

; ------------------ Internal memory usage page 7 -------------------
; ------------------  (380h-3FFh = B1 upper half) -------------------


; ------------------ External memory usage page 8 -------------------

; most of the memory accesses go to this page while running
; BUFI/BUFO goto page 0 during INIT and to page 8 during run

CFG       .EQU      02h       ;  hold AIO configuration

MON0      .EQU      06h       ;  monitor scratch register

MON1      .EQU      060h      ;  monitor scratch register
MON2      .EQU      061h      ;  monitor scratch register
MON3      .EQU      062h      ;  monitor scratch register
;BUFI     .EQU      07Ch      ;  input from AIO during run-time
;BUFO     .EQU      07Dh      ;  output to AIO during run-time

SAMP      .EQU      015h      ;  current analog sample to output
PHASE     .EQU      016h      ;  current phase postion (NCO accumulator)
TABLE     .EQU      017h      ;  holds address of output waveform table
PHASEB    .EQU      018h      ;  holds phase >> 8
TEMP      .EQU      419h      ;  holds input from SIO
PARM      .EQU      41Ah      ;  Parameters from Windows.  Takes 4 words.
AMPL      .EQU      01Ah      ;  PARM[0] = Output amplitude
FREQ      .EQU      01Bh      ;  PARM[1] = Output frequency
WAVE      .EQU      01Ch      ;  PARM[2] = Output waveform type
DUMMY     .EQU      01Dh      ;  PARM[3] = Dummy value

SAVEST0   .EQU      02Ah      ;  save ST0 during interrupt
SAVEST1   .EQU      02Bh      ;  save ST1 during interrupt


; ----------------- Parameter values and constants -------------------

CMDA      .EQU      0810h     ;  RA/TA = 4
CMDB      .EQU     02852h     ;  AIO load RB/TB = 20 for 62,500 samples/sec
CMDC      .EQU       0A3h     ;  AIO config: A/D gain 2, sync, no HPF or sinc corr
BASE      .EQU       020h     ;  Pointer to waveform TABLES<<8
			      ;  (points to 2000h when shifted by 8 bits)
			      ;  Table 0 at 2000h, table 1 at 2100h, 
			      ;  table 2 at 2200h, table 3 at 2300h


	  .ORG      1000h


	  B         GO        ;  branch to program start (Timer Intrpt)
	  B         RINT      ;  SIO receive interrupt (AIO-rx'd value)
	  B         XINT      ;  SIO transmit interrupt (AIO-tx'd value)
	  B         GO        ;  branch to program start (SW Trap)


GO        DINT                ;  disable interrupts
	  LDPK      08h       ;  data mem page 8 (external)

MATH_INI  SSXM                ;  set sign extension mode
	  SOVM                ;  saturate on overflow
	  SPM       00h       ;  set P shift = 0

AIO_RST   LALK      8025h     ;  reset D15 to disconnect AIO (DX&DR)
	  SACL      CFG       ;  clear D14 to hold AIO in RESET mode
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      0065h     ;  load AIO reset, radio 1, gain = 2
	  SACL      CFG       ;  store in AIO CFG buffer
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

	  LALK      4025h     ;  load AIO run, radio 1, gain = 2
	  SACL      CFG       ;  store in AIO CFG buffer
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

	  LDPK      00h       ;  data mem page 0 (internal)

	  SFSM                ;  frame sync SIO to AIO
	  RTXM                ;  make FSX a CPU input

	  LALK      0020h     ;  mask SIO TX interrupt active
	  SACL      IMR       ;  store INT mask

	  ZAC                 ;  zero A
	  SACL      BUFO      ;  store in BUFO
	  EINT                ;  enable TX interrupt
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDC      ;  load AIO config command
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDB      ;  load TB/RB AIO command next
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDA      ;  load TA/RA AIO command last!!
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  DINT                ;  disable interrupts
	  LALK      0020h     ;  allow AIO TX interrupts only
	  SACL      IMR       ;  store mask in IMR
INI_END   CNFD                ;  set block B0 as data memory

	  LDPK      8         ;  point to external RAM

	  LRLK      AR0,2000h ;  AR0 points to 2000h in data memory
	  LARP      AR0

	  LALK      WTABLE+000h ;  Accum points to Sin waveform table in PROG mem
	  RPTK      0ffh      ;  repeat instruction 256 times
	  TBLR      *+        ;    Move waveform table to data memory

	  LALK      WTABLE+100h ;  Accum points to Tri waveform table in PROG mem
	  RPTK      0ffh      ;  repeat instruction 256 times
	  TBLR      *+        ;    Move waveform table to data memory

	  LALK      WTABLE+200h ;  Accum points to Sqr waveform table in PROG mem
	  RPTK      0ffh      ;  repeat instruction 256 times
	  TBLR      *+        ;    Move waveform table to data memory

	  LALK      WTABLE+300h ;  Accum points to Saw waveform table in PROG mem
	  RPTK      0ffh      ;  repeat instruction 256 times
	  TBLR      *+        ;    Move waveform table to data memory

	  EINT                ;  enable transmit interrupts


WAIT_RX   ; wait for rcvd chars from Windows:
	  ; Start Command, Amplitude, Frequency, Wavetype, Dummy
	  ; or Stop command

	  LRLK      AR0, TEMP ; point to temp receive char
	  LRLK      AR1, PARM ; point to PARM block
	  LARK      AR2, 3    ; get 4 words (8 bytes)
	
RX1       LARP      AR0
	  CALL      INBIT     ; get character from SERIAL to TEMP
	  LAC       *         ; char to accum
	  ANDK      0FFh
	  SUBK      'P'       ; compare against rcv'd char
	  BZ        RX5       ; if 'P' then start

	  LAC       *         ; get same char back to accum again
	  ANDK      0FFh
	  SUBK      'R'       ; compare against rcv'd char
	  BNZ       RX1       ; if not 'R' then wait for next char

	  CALL      RESET     ; got 'R' so oscilloscope is done

; have received starting flag, load next 8 bytes (4 words) to PARM buffer

RX5       LARP      AR1       ; point to PARM[0]
	  ZAC
	  RPTK      3
	  SACL      *+        ; PARM[0..3] = 0
	  LRLK      AR1, PARM ; reload AR1

RX2       LARP      AR0       ; point to TEMP
	  ZAC
	  SACL      *         ; TEMP = 0
	  LARK      AR3, 3    ; 4 ASCII characters = one 16-bit word

RX3       LARP      AR0
	  CALL      INBIT     ; get char to TEMP
	  LAC       *         ; load char to accum
	  ANDK      0FFh      ; clear high-order bits
	  SUBK      3Ah       ; if (char <= '9')
	  BGEZ      RX4       ; 
	  ADDK      07h       ;   then adjust ASCII 0..9 to numeric
RX4       ADDK      03h       ;   else adjust ASCII A..F to numeric

	  LARP      AR1       ; point to PARM[n]
	  RSXM                ; no sign-extension
	  ADD       *,4       ; shift up one nibble then add new char
	  SACL      *         ; store back to PARM[n]

	  LARP      AR3       ; load nibble counter
	  BANZ      RX3, *-   ; do next nibble

	  LARP      AR1
	  ADRK      1         ; Increment PARM pointer
	  LARP      AR2       ; load word counter
	  BANZ      RX2,*-    ; do next word

	  B         WAIT_RX   ; wait and get parameters again
			      ; TX interrupts are active

; ------------------------
;  the parameters ranges...
;
;  Amplitude    : 0..7fffh      sets output amplitude
;  Frequency    : 1..7fffh      sets NCO accumulator rate
;  WaveType     : 0 = sine wave
;               : 1 = square wave
;               : 2 = triangle wave
;               : 3 = sawtooth wave
;




	  
; -------------------- AIO received interrupt handler ---------------

RINT      EINT                ;  more samples to get - enable interrupts
	  RET                 ;  and return



; -------------------- AIO transmitted interrupt handler ------------

XINT
	  DINT
	  PUSH                ; save Accumulator

	  LARP      AR7       ; ARP=AR7   ARB=old ARP
	  LRLK      AR7,SAVEST1+400h ; point AR7 to save area for ST
	  SST1      *-        ; save ST1
	  SST       *         ; save ST0

	  RSXM                ; clear sign-extension mode

	  LAC       PHASE     ; get current phase
	  ADD       FREQ      ; add new phase increment
	  SACL      PHASE     ; store new phase 

	  RC                  ; clear carry bit
	  RPTK      7         ; repeat instruction 8 times
	  ROR                 ;   rotate down 8 bits
	  ANDK      0FFh      ; get rid of any bits from High Accum
	  SACL      PHASEB    ; store phase >> 8

	  LAC       WAVE      ; get waveform type (0,1,2,3)
	  ADDK      BASE      ; accum = base addr of one of the four tables
	  SACL      TABLE     ; store table base
	  LAC       TABLE,8   ; retrieve table base, shifted to point to page
	  ADD       PHASEB    ; add phase index to table
	  SACL      TABLE     ; TABLE = &table[waveform][phaseb]

	  LARP      AR7
	  LAR       AR7,TABLE ; AR7 points to a waveform table sample

	  SSXM                ; set sign-extension mode

	  LT        *         ; load amplitude into T
	  MPY       AMPL      ; output = sample * amplitude_scale_factor
	  SPH       BUFO      ; put most significant part of output to BUFO
	  
	  LARK      AR7,DXR   ; point AR7 at AIO output register
	  LAC       BUFO,1    ; load accum from waiting output (scale by two)
	  SACL      *         ; send to AIO chip

	  LRLK      AR7,SAVEST0+400h ; point AR7 to save area for ST
	  LST       *+        ; restore ST0
	  LST1      *         ; restore ST1, reload ARP=old ARP
	  POP                 ; restore accumulator

	  EINT                ; enable interrupts
	  RET



; --------------------- Time delay subroutines ----------------------

WAIT4     LARK      AR2,0FFh
	  CALL      WAIT2
	  CALL      WAIT2

	  RET

WAIT2     LARK      AR2,0FFh
	  CALL      WAIT1
	  CALL      WAIT1

	  RET

WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
	  BANZ      WAIT_B,*-  ; Loop through count

	  LARP      AR2
	  BANZ      WAIT_A,*-  ; Loop through count

	  RET


; ----------------------- waveform tables -------------------------
;
; Converts phase position (0..255) to amplitude of that phase position
;
; Contains 4 tables, each 256 words long.
;
; Table 0 = Sine(theta)
; Table 1 = Triangle(theta)
; Table 2 = Square(theta)
; Table 3 = Sawtooth(theta)

	.MSFIRST

WTABLE          ; address of wavetable data in memory

#include        "wavetabl.dat"



	  .END
