;                                               D93SCOPE.ASM
;
;
;  Digital sampling oscilloscope for DSP-93 and D93WE Windows program  
;
; Copyright (c) 1994 by Tucson Amateur Packet Radio Corporation
;
; This document, and its executable, are the property of the Tucson Amateur
; Packet Radio Corp., a Arizona non-profit corporation, and all are copyright
; 1994.  Permission is hereby granted for the non-commercial use of these
; properties only by licensed amateur radio operators.  All other rights are
; reserved  by the Tucson Amateur Packet Radio Corp. 
;
; This software is provided for use only to non-commercial amateur radio
; users.  Commercial inquiries should be made to Tucson Amateur Packet Radio
; Corp.
;
; TAPR (Tucson Amateur Packet Radio Corporation) makes no warranty of any
; kind, express or implied, including without limitation, any warranties
; and/or fitness for a particular purpose of this software.  TAPR shall not be
; liable for any damages, whether direct, indirect, special or consequential
; arising from a failure of this software to operate in the manner desired by
; the user.  TAPR shall not be liable for any damage to data or property that
; may be caused directly or indirectly by using the software.
;
; IN NO EVENT WILL TUCSON AMATEUR PACKET RADIO CORP. BE LIABLE FOR ANY
; INCIDENTAL OR CONSEQUENTIAL DAMAGE ARISING OUT OF USE OR INABILITY TO USE
; THE PROGRAM, OR FOR ANY CLAIM BY ANY OTHER PARTY.
;
;
;  Revision history
;
;  Version   Date     Author     Reason
;  ------- -------- -----------  ---------------------------------------------------------
;  0.3.5   10-17-94 TCMcDermott  Prototype #3 release
;  0.3.6   10-23-94 TCMcDermott  Add 'Q' at end of block in case PC misses
;                                  an Rx character.  Change from regs.asm,
;				   monitor.asm, to regs.inc, monitor.inc
;  1.0     12-11-94 TCMcDermott  Production #1 release.
;				 Move page 8 memory references to avoid
;				 overlap with monitor 2.15.
;
;
;
;  This version programs the AIO for maximum sampling rate (41.666 ksps)
;  (40 Mhz CPU clock).
;  AIO initialization from Lon Cecil's BST5.ASM program
;
;  The algorithm used is:
;
;  1. Init the DSP-93 AIO and serial code 
;  2. loop: wait-for-RX-ser-char
;  3. D93WE will send 'P', trigger-level, trig-slope, gain, and sweep
;  4. DSP-93 goes into a loop waiting for trigger condition
;  5. If DSP-93 times out waiting for sweep, then it will auto-trigger
;  6. When DSP-93 triggers, it will capture a full sample buffer
;  7. DSP-93 will then send the buffer to the PC over the serial port
;  8. Go to step 2


#include        "regs.inc"
#include        "monitor.inc"

; ------------------ Internal memory usage page 0 -------------------
; ------------------ only valid for 60h-7Fh = B2  -------------------

; only using page 0 during the AIO configuration setup

BUFI      .EQU      7Ch       ;  hold input from AIO chip for INIT
BUFO      .EQU      7Dh       ;  hold output to AIO chip for INIT

; ------------------ Internal memory usage page 4 -------------------
; ---------  (200h-27Fh = B0 lower half when B0 is data)  -----------

; ------------------ Internal memory usage page 5 -------------------
; ---------  (280h-2FFh = B0 upper half when B0 is data)  -----------

; ------------------ Internal memory usage page 6 -------------------
; ------------------  (300h-37Fh = B1 lower half) -------------------

; ------------------ Internal memory usage page 7 -------------------
; ------------------  (380h-3FFh = B1 upper half) -------------------


; ------------------ External memory usage page 8 -------------------

; most of the memory accesses go to this page while running
; BUFI/BUFO goto page 0 during INIT and to page 8 during run

CFG       .EQU      02h       ;  hold AIO configuration
MON0      .EQU      06h       ;  monitor scratch register
MON1      .EQU      060h      ;  monitor scratch register
MON2      .EQU      061h      ;  monitor scratch register
MON3      .EQU      062h      ;  monitor scratch register
;BUFI     .EQU      07Ch      ;  input from AIO during run-time
;BUFO     .EQU      07Dh      ;  output to AIO during run-time

TRIGL     .EQU      015h      ;  binary value of trigger level
TRIGS     .EQU      016h      ;  trig slope 0001=+ FFFF=-
CURSAMP   .EQU      017h      ;  current analog sample
PRESAMP   .EQU      018h      ;  previous analog sample
ARMED     .EQU      019h      ;  0=triggered  1=not_triggered
TEMP      .EQU      41Ah      ;  holds input from SIO
PARM      .EQU      41Bh      ;  Parameters from Windows.  Takes 5 words.
SWEEP     .EQU      01Dh      ;  PARM[2] = Sweep speed
GAIN      .EQU      01Eh      ;  PARM[3] = Gain setting
AUTOTR    .EQU      01Fh      ;  PARM[4] = Auto Trigger Flag


; ----------------- Parameter values and constants -------------------

CMDA      .EQU      0810h     ; RA/TA = 4   => 8.2 khz filter cutoff (32044)
CMDB      .EQU      02852h    ;  AIO load RB/TB = 20 for 62,500 samples/sec
CMDC      .EQU      0A3h      ;  AIO config: A/D gain 2, sync, no HPF or sinc corr


	  .ORG      1000h


	  B         GO        ;  branch to program start (Timer Intrpt)
	  B         RINT      ;  SIO receive interrupt (AIO-rx'd value)
	  B         XINT      ;  SIO transmit interrupt (AIO-tx'd value)
	  B         GO        ;  branch to program start (SW Trap)


GO        DINT                ;  disable interrupts
	  LDPK      08h       ;  data mem page 8 (external)


MATH_INI  SSXM                ;  set sign extension mode
	  SOVM                ;  saturate on overflow
	  SPM       00h       ;  set P shift = 0

AIO_RST   LALK      8025h     ;  reset D15 to disconnect AIO (DX&DR)
	  SACL      CFG       ;  clear D14 to hold AIO in RESET mode
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

AIO_INI   LALK      0065h     ;  load AIO reset, radio 1, gain = 2
	  SACL      CFG       ;  store in AIO CFG buffer
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

	  LALK      401Dh     ;  load AIO run, radio 1, gain = 2
	  SACL      CFG       ;  store in AIO CFG buffer
	  OUT       CFG,07h   ;  OUT to config port

	  CALL      WAIT4     ;  delay 400 ms

	  LDPK      00h       ;  data mem page 0 (internal)

	  SFSM                ;  frame sync SIO to AIO
	  RTXM                ;  make FSX a CPU input

	  LALK      0020h     ;  mask SIO TX interrupt active
	  SACL      IMR       ;  store INT mask

	  ZAC                 ;  zero A
	  SACL      BUFO      ;  store in BUFO
	  EINT                ;  enable TX interrupt
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDC      ;  load AIO config command
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDB      ;  load TB/RB AIO command next
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      0003h     ;  load AIO command flag
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  LALK      CMDA      ;  load TA/RA AIO command last!!
	  SACL      BUFO      ;  store in BUFO
	  IDLE                ;  wait for TX interrupt

	  DINT                ;  disable interrupts
	  LALK      0010h     ;  allow AIO RX interrupts only
	  SACL      IMR       ;  store mask in IMR
INI_END   CNFD                ;  set block B0 as data memory


WAIT_RX   ; wait for rcvd chars from Windows:
	  ; Trigger-val, trigger-slope, gain, sweep-rate,
	  ; auto-trigger, start command

	  LDPK      8         ; point to external RAM

	  LRLK      AR0, TEMP ; point to temp receive char
	  LRLK      AR1, PARM ; point to PARM block
	  LARK      AR2, 4    ; get 5 parameters (20 characters)
	
RX1       LARP      AR0
	  CALL      INBIT     ; get character from SERIAL to TEMP
	  LAC       *         ; char to accum
	  ANDK      0FFh

	  SUBK      'P'       ; compare against rcv'd char
	  BZ        RX5       ; if 'P' then start

	  LAC       *         ; get same char back to accum again
	  ANDK      0FFh

	  SUBK      'R'       ; compare against rcv'd char
	  BNZ       RX1       ; if not 'R' then wait for next char

	  CALL      RESET     ; got 'R' so oscilloscope is done


; have received starting flag, load next 20 chars (5 words) to PARM buffer

RX5       LARP      AR1       ; point to PARM[0]
	  ZAC
	  RPTK      4
	  SACL      *+        ; PARM[0..4] = 0
	  LRLK      AR1, PARM ; reload AR1

RX2       LARP      AR0       ; point to TEMP
	  ZAC
	  SACL      *         ; TEMP = 0
	  LARK      AR3, 3    ; 4 ASCII characters = one 16-bit word

RX3       LARP      AR0
	  CALL      INBIT     ; get char to TEMP
	  LAC       *         ; load char to accum
	  ANDK      0FFh      ; clear high-order bits
	  SUBK      3Ah       ; if (char <= '9')
	  BGEZ      RX4       ; 
	  ADDK      07h       ;   then adjust ASCII 0..9 to numeric
RX4       ADDK      03h       ;   else adjust ASCII A..F to numeric

	  LARP      AR1       ; point to PARM[n]
	  RSXM                ; no sign-extension
	  ADD       *,4       ; shift up one nibble then add new char
	  SACL      *         ; store back to PARM[n]

	  LARP      AR3       ; load nibble counter
	  BANZ      RX3, *-   ; do next nibble

	  LARP      AR1
	  ADRK      1         ; Increment PARM pointer
	  LARP      AR2       ; load word counter
	  BANZ      RX2,*-    ; do next word

; ------------------------
;  Now process the parameters
;
;  Sweep, Gain  = Integer values for Sweep, Gain
;  Slope        :          0001 = +  
;               : anything else = -
;  Triger level = 16-bit binary value
;  Auto Trigger = Integer flag for Auto Trigger Enable
;               :          0000 = Disabled
;               : anything else = Enabled
;

	  LRLK      AR1, PARM ; point to trigger level
	  LARP      AR1
	  LAC       *+ 
	  SACL      TRIGL     ; store trig level
	  SACL      PRESAMP   ; and store as prev sample
	  LAC       *+        ; get trig slope
	  SACL      TRIGS     ; and store it
	  LACK      1         ; 1=not_armed, 0=armed
	  SACL      ARMED     ; set not_armed

	  LAC       GAIN      ; gain word (already coded)
	  SACL      CFG       ; store in AIO CFG buffer
	  OUT       CFG,07h   ; modify DSP-93 analog gain setting

	  LRLK      AR2, 01FFh ; 512 samples
	  LRLK      AR3, 2000h ; our received sample buffer space
	  LRLK      AR4, 1000h ; autotrigger timeout
	  LRLK      AR5, DRR   ; point to RXDATA in page zero
	  LAR       AR6, SWEEP ; load sweep counter
	  SSXM                ; set sign-extension mode

	  EINT                ; enable rcv'd interrupts

ML1       LARP      AR2
	  BANZ      ML1,*     ; wait for RX Interrupt to decr AR2


; ------------------------
;  Send the captured binary blocks to D93WE
;       as ASCII in the form  hhhh

	  DINT

	  LRLK      AR2, 1FFh ; 512 samples
	  LRLK      AR3, 2000h ; located in data buffer

ML2       LARP      AR3       ; point to received samples
	  LAC       *+        ; load next sample to Accum
	  SACL      MON0      ; store it where PRVAL wants it
	  CALL      PRVAL16   ; send sample as hhhh
	  LARP      AR2       ; get block count
	  BANZ      ML2, *-   ; and do next sample

	  LACK      'Q'
	  SACL      MON3
	  CALL      OUTBIT2   ; send 'Q' as end-of-buffer marker

	  B         WAIT_RX   ; go wait for D93WE to kick us off again

	  
; -------------------- AIO received interrupt handler ---------------

RINT
	  LARP      AR6       ;  get sweep counter
	  BANZ      RI01,*-   ;  if not zero, then don't get sample

	  LAR       AR6,SWEEP ;  valid sample time - reload sweep counter
	  LARP      AR5       ;  AR5 points at DRR
	  LAC       *         ;  get received AIO data sample from DRR
	  SACL      CURSAMP   ;  store DDR in AIO input buffer
	  LAC       ARMED     ;  are we triggered?
	  BNZ       RINT1     ;  jif if not

; we are triggered, so get a sample

	  LARP      AR3       ;  get pointer to data buffer
	  LAC       CURSAMP   ;  get current sample
	  SACL      *+        ;  store value, and incr pointer
	  LARP      AR2       ;  get sample size
	  BANZ      RI02,*-   ;  decrement samples remaining
	  RET                 ;  return leaving interrupts disabled

RI01      LARP      AR2       ;  make our caller happy
			      ;  fall through - return from int
RI02      EINT                ;  more samples to get - enable interrupts
	  RET                 ;  and return

; we are not triggered, see if new sample triggers us

RINT1     LAC       TRIGS     ;  get trigger slope (0001=positive)
	  SUBK      1         ;  is it negative?
	  BZ        RINT2     ;  jif positive slope

	  LAC       PRESAMP   ;  we are negative slope
	  SUB       TRIGL     ;  if (PREVSAMP <= TRIGL)
	  BLEZ      RINT3     ;     then branch  (No-Trigger)
	  LAC       CURSAMP   ;
	  SUB       TRIGL     ;  if (CURSAMP > TRIG)
	  BGZ       RINT3     ;     then branch  (No-Trigger)
	  B         RINT4     ;  we are triggered

RINT2     LAC       PRESAMP   ;  we are positive slope
	  SUB       TRIGL     ;  if (PREVSAMP >= TRIG)
	  BGEZ      RINT3     ;     then branch (No-Trigger)
	  LAC       CURSAMP   ;
	  SUB       TRIGL     ;  if (CURSAMP <= TRIG)
	  BLZ       RINT3     ;     then branch (No-Trigger)
			      ;  PREVSAMP<TRIGL && CURSAMP>TRIGL,
	  B         RINT4     ;    so we are triggered
 
RINT3     LARP      AR4       ;  not triggered
	  BANZ      RINT5, *- ;  decrement auto-trigger timeout

; Auto trigger counter decremented to zero.
; If AutoTr is set, then allow auto trigger, else don't trigger

	  LAC       AUTOTR    ;  get Auto Trigger Enable flag
	  BZ        RINT5     ;  if =0, then don't autotrigger  

RINT4     ZAC                 ;  auto-trigger reached zero or triggered
	  SACL      ARMED     ;  we are now triggered
RINT5     LARP      AR2       ;  make our caller happy
	  LAC       CURSAMP
	  SACL      PRESAMP   ;  shift sample to old
	  EINT
	  RET




; -------------------- AIO transmitted interrupt handler ------------

XINT      
	  LAC       BUFO      ;  load accum from waiting output
	  SACL      DXR       ;  send to AIO chip

	  EINT                
	  RET


; --------------------- Time delay subroutines ----------------------

WAIT4     LARK      AR2,0FFh
	  CALL      WAIT2
	  CALL      WAIT2

	  RET

WAIT2     LARK      AR2,0FFh
	  CALL      WAIT1
	  CALL      WAIT1

	  RET

WAIT1     LARK      AR2,0FFh
WAIT_A    LARK      AR3,0FFh
WAIT_B    LARP      AR3
	  BANZ      WAIT_B,*-  ; Loop through count

	  LARP      AR2
	  BANZ      WAIT_A,*-  ; Loop through count

	  RET

	  .END
